"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.afterEach = exports.beforeEach = void 0;
const format_js_1 = require("./format.js");
const write_js_1 = require("../services/write.js");
const output_js_1 = require("../parsers/output.js");
const eachCore = (type, fileRelative, configs) => __awaiter(void 0, void 0, void 0, function* () {
    /* c8 ignore next 3 */
    if (typeof (configs === null || configs === void 0 ? void 0 : configs[type]) !== 'function') {
        return true;
    }
    const cb = configs[type];
    const showLogs = !(0, output_js_1.isQuiet)(configs);
    const cbName = cb.name !== type ? cb.name : 'anonymous function';
    showLogs &&
        write_js_1.Write.log(`    ${(0, format_js_1.format)('◯').dim().info()} ${(0, format_js_1.format)(`${type}: ${cbName}`)
            .dim()
            .italic()}`);
    try {
        yield cb();
        return true;
    }
    catch (error) {
        if (showLogs) {
            write_js_1.Write.log((0, format_js_1.format)(`    ✘ ${type} callback failed ${(0, format_js_1.format)(`› ${cbName}`).dim()}`)
                .fail()
                .bold());
            write_js_1.Write.log((0, format_js_1.format)(`      ├─ Who's trying to run this ${type}?`).fail());
            write_js_1.Write.log((0, format_js_1.format)(`      │ └─ ${(0, format_js_1.format)(fileRelative).fail().underline()}`).fail());
            if (error instanceof Error) {
                write_js_1.Write.log((0, format_js_1.format)('      ├─ Message:').fail());
                write_js_1.Write.log((0, format_js_1.format)(`      │ └─ ${error.message}`).fail());
            }
        }
        return false;
    }
});
const beforeEach = (fileRelative, configs) => __awaiter(void 0, void 0, void 0, function* () {
    if (configs === null || configs === void 0 ? void 0 : configs.beforeEach) {
        return yield eachCore('beforeEach', fileRelative, configs);
    }
    return true;
});
exports.beforeEach = beforeEach;
/* c8 ignore next */ // ?
const afterEach = (fileRelative, configs) => __awaiter(void 0, void 0, void 0, function* () {
    if (configs === null || configs === void 0 ? void 0 : configs.afterEach) {
        return yield eachCore('afterEach', fileRelative, configs);
    }
    return true;
});
exports.afterEach = afterEach;
